@@ -21,11 +21,11 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/param.h> /* howmany() */
 
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#include "bidi.hh"
 #include "vtedraw.hh"
 #include "vtedefines.hh"
 #include "debug.h"
@@ -381,18 +381,32 @@ font_info_measure_font (struct font_info *info)
 {
 	PangoRectangle logical;
 
-	/* Estimate for ASCII characters. */
+        /* Measure U+0021..U+007E individually instead of all together and then
+         * averaging. For monospace fonts, the results should be the same, but
+         * if the user (by design, or trough mis-configuration) uses a proportional
+         * font, the latter method will greatly underestimate the required width,
+         * leading to unreadable, overlapping characters.
+         * https://gitlab.gnome.org/GNOME/vte/issues/138
+         */
+        int max_width{1};
+        int max_height{1};
+        for (char c = 0x21; c < 0x7f; ++c) {
+                pango_layout_set_text(info->layout, &c, 1);
+                pango_layout_get_extents (info->layout, NULL, &logical);
+                max_width = std::max(max_width, PANGO_PIXELS_CEIL(logical.width));
+                max_height = std::max(max_height, PANGO_PIXELS_CEIL(logical.height));
+        }
+
+        /* Use the sample text to get the baseline */
 	pango_layout_set_text (info->layout, VTE_DRAW_SINGLE_WIDE_CHARACTERS, -1);
 	pango_layout_get_extents (info->layout, NULL, &logical);
 	/* We don't do CEIL for width since we are averaging;
 	 * rounding is more accurate */
-	info->width  = PANGO_PIXELS (howmany (logical.width, strlen(VTE_DRAW_SINGLE_WIDE_CHARACTERS)));
-        /* Guard against pathological font since width=0 causes a FPE later on */
-	info->width = MAX (info->width, 1);
-
-	info->height = PANGO_PIXELS_CEIL (logical.height);
 	info->ascent = PANGO_PIXELS_CEIL (pango_layout_get_baseline (info->layout));
 
+        info->height = max_height;
+        info->width = max_width;
+
 	/* Now that we shaped the entire ASCII character string, cache glyph
 	 * info for them */
 	font_info_cache_ascii (info);
@@ -812,6 +826,22 @@ _vte_draw_set_cairo (struct _vte_draw *draw,
         }
 }
 
+void
+_vte_draw_clip(struct _vte_draw *draw,
+               cairo_rectangle_int_t const* rect)
+{
+        cairo_save(draw->cr);
+        cairo_rectangle(draw->cr,
+                        rect->x, rect->y, rect->width, rect->height);
+        cairo_clip(draw->cr);
+}
+
+void
+_vte_draw_unclip(struct _vte_draw *draw)
+{
+        cairo_restore(draw->cr);
+}
+
 static void
 _vte_draw_set_source_color_alpha (struct _vte_draw *draw,
                                   vte::color::rgb const* color,
@@ -1471,13 +1501,20 @@ _vte_draw_text_internal (struct _vte_draw *draw,
 
 	for (i = 0; i < n_requests; i++) {
 		vteunistr c = requests[i].c;
+
+                if (G_UNLIKELY (requests[i].mirror)) {
+                        vte_bidi_get_mirror_char (c, requests[i].box_mirror, &c);
+                }
+
 		struct unistr_info *uinfo = font_info_get_unistr_info (font, c);
 		union unistr_font_info *ufi = &uinfo->ufi;
                 int x, y;
 
                 _vte_draw_get_char_edges(draw, c, requests[i].columns, style, &x, NULL);
                 x += requests[i].x;
-                y = requests[i].y + draw->char_spacing.top + font->ascent;
+                /* Bold/italic versions might have different ascents. In order to align their
+                 * baselines, we offset by the normal font's ascent here. (Bug 137.) */
+                y = requests[i].y + draw->char_spacing.top + draw->fonts[VTE_DRAW_NORMAL]->ascent;
 
                 if (_vte_draw_unichar_is_local_graphic(c)) {
                         _vte_draw_terminal_draw_graphic(draw, c, color,
